<?php

namespace Drupal\radar\Form;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;

use Drupal\Core\State\StateInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\file\FileInterface;
use Drupal\file\FileStorageInterface;
use Drupal\stream_wrapper_example\SessionHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;


/**
 * Implements the modal form.
 *
 * This example demonstrates implementation of a form that is designed to be
 * used as a modal form.  To properly display the modal the link presented by
 * the \Drupal\form_api_example\Controller\Page page controller loads the Drupal
 * dialog and ajax libraries.  The submit handler in this class returns ajax
 * commands to replace text in the calling page after submission .
 *
 * @see \Drupal\Core\Form\FormBase
 */
class RadarModalForm extends FormBase {

    /**
   * Interface of the "state" service for site-specific data.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Service for manipulating a file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Service for fetching a stream wrapper for a file or directory.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * The session helper.
   *
   * @var \Drupal\stream_wrapper_example\SessionHelper
   */
  protected $sessionHelper;

  /**
   * The dumper.
   *
   * @var \Drupal\devel\DevelDumperManagerInterface
   */
  protected $dumper;

  /**
   * The file entity storage.
   *
   * @var \Drupal\file\FileStorageInterface
   */
  protected $fileStorage;

  /**
   * Constructs a new FileExampleReadWriteForm page.
   *
   * @param \Drupal\Core\State\StateInterface $state
   *   Storage interface for state data.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   Interface for common file system operations.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $stream_wrapper_manager
   *   Interface to obtain stream wrappers used to manipulate a given file
   *   scheme.
   * @param \Drupal\file\FileStorageInterface $file_storage
   *   The file entity storage.
   * @param \Drupal\stream_wrapper_example\SessionHelper $session_helper
   *   Session helper.
   */
  public function __construct(
    StateInterface $state,
    FileSystemInterface $file_system,
    StreamWrapperManagerInterface $stream_wrapper_manager,
    FileStorageInterface $file_storage
  ) {
    $this->state = $state;
    $this->fileSystem = $file_system;
    $this->streamWrapperManager = $stream_wrapper_manager;
    $this->fileStorage = $file_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $form = new static(
      $container->get('state'),
      $container->get('file_system'),
      $container->get('stream_wrapper_manager'),
      $container->get('entity_type.manager')->getStorage('file')
    );
    $form->setRequestStack($container->get('request_stack'));
    $form->setStringTranslation($container->get('string_translation'));
    $form->setMessenger($container->get('messenger'));
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'radar_modal_form';
  }

  /**
   * Helper method so we can have consistent dialog options.
   *
   * @return string[]
   *   An array of jQuery UI elements to pass on to our dialog form.
   */
  protected static function getDataDialogOptions() {
    return [
      'width' => '70%',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $nojs = NULL) {
    $form['description'] = [
      '#type' => 'item',
      '#markup' => $this->t('Use one sentence to tell us what\'s the issue is:'),
    ];

    // This element is responsible for displaying form errors in the AJAX
    // dialog.
    $form['status_messages'] = [
      '#type' => 'status_messages',
      '#weight' => -999,
    ];

    $form['screenshot'] = [
      '#type' => 'hidden',
      '#weight' => -999,
    ];

    $form['issue'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Issue summary'),
      '#required' => TRUE,
    ];

    // Group submit handlers in an actions element with a key of "actions" so
    // that it gets styled correctly, and so that other modules may add actions
    // to the form.
    $form['actions'] = [
      '#type' => 'actions',
    ];

    // Add a submit button that handles the submission of the form.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#ajax' => [
        'callback' => '::ajaxSubmitForm',
        'event' => 'click',
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // $issue = $form_state->getValue('issue');
    // $url = $form_state->getValue('screenshot');
    // $screenshot = "<br /><img src=\"data:".$url."\" />...";
    // \Drupal::logger("User Feedback")->error($issue . $screenshot);
  }

  /**
   * Implements the submit handler for the modal dialog AJAX call.
   *
   * @param array $form
   *   Render array representing from.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Current form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Array of AJAX commands to execute on submit of the modal form.
   */
  public function ajaxSubmitForm(array &$form, FormStateInterface $form_state) {
    // We begin building a new ajax reponse.
    $response = new AjaxResponse();

    // If the user submitted the form and there are errors, show them the
    // input dialog again with error messages. Since the title element is
    // required, the empty string wont't validate and there will be an error.
    if ($form_state->getErrors()) {
      // If there are errors, we can show the form again with the errors in
      // the status_messages section.
      $form['status_messages'] = [
        '#type' => 'status_messages',
        '#weight' => -10,
      ];
      $response->addCommand(new OpenModalDialogCommand($this->t('Errors'), $form, static::getDataDialogOptions()));
    }
    // If there are no errors, show the output dialog.
    else {
      // We don't want any messages that were added by submitForm().
      $this->messenger()->deleteAll();
      // We use FormattableMarkup to handle sanitizing the input.
      // @todo: There's probably a better way to do this.
      $issue = new FormattableMarkup(':issue', [':issue' => $form_state->getValue('issue')]);
      // This will be the contents for the modal dialog.
      $content = [
        '#type' => 'item',
        '#markup' => $this->t("We've got your report already. Thank you!"),
      ];
      // Add the OpenModalDialogCommand to the response. This will cause Drupal
      // AJAX to show the modal dialog. The user can click the little X to close
      // the dialog.
      $response->addCommand(new OpenModalDialogCommand($issue, $content, static::getDataDialogOptions()));
      $data = $form_state->getValue('screenshot');

      /*
      // NOT working.
      $screenshot = [
        '#attributes' => [],
        '#data' => $data, //new FormattableMarkup('<img src="data::src"></img>', [':src' => $url]),
        '#theme_wrappers' => [
           'image_base64',
         ],
      ];
      // NOT working.
      $screenshot = [
        '#type' => 'inline_template',
        '#template' => "<img src='data:image/png;base64,{{data}}' />", //new FormattableMarkup('<img src="data::src"></img>', [':src' => $url]),
        '#context' => [
           'data' => $data
         ],
      ];
    */
      $data = base64_decode(str_replace("data:image/png;base64,",'', $data));
      $uri = 'public://' . time();

      // Managed operations work with a file object.
      $file_object = \file_save_data($data, $uri, FileSystemInterface::EXISTS_RENAME);
      if (!empty($file_object)) {
        $url = $this->getExternalUrl($file_object);
        $screenshot = Markup::create("<img src='".$url->toString()."' />");
        \Drupal::logger("User Feedback")->error($issue . $screenshot);
      }
      else {
        $this->messenger()->addMessage($this->t('Failed to save the managed file'), 'error');
      }

      // \Drupal::logger("User Feedback")->error($issue . \Drupal::service('renderer')->render($screenshot));
    }

    // Finally return our response.
    return $response;
  }

    /**
   * Prepare Url objects to prevent exceptions by the URL generator.
   *
   * Helper function to get us an external URL if this is legal, and to catch
   * the exception Drupal throws if this is not possible.
   *
   * In Drupal 8, the URL generator is very sensitive to how you set things
   * up, and some functions, in particular LinkGeneratorTrait::l(), will throw
   * exceptions if you deviate from what's expected. This function will raise
   * the chances your URL will be valid, and not do this.
   *
   * @param \Drupal\file\Entity\File|string $file_object
   *   A file entity object.
   *
   * @return \Drupal\Core\Url
   *   A Url object that can be displayed as an internal URL.
   */
  protected function getExternalUrl($file_object) {
    if ($file_object instanceof FileInterface) {
      $uri = $file_object->getFileUri();
    }
    else {
      // A little tricky, since file.inc is a little inconsistent, but often
      // this is a Uri.
      // $uri = file_create_url($file_object);
    }

    try {
      // If we have been given a PHP stream URI, ask the stream itself if it
      // knows how to create an external URL.
      $wrapper = $this->streamWrapperManager->getViaUri($uri);
      if ($wrapper) {
        $external_url = $wrapper->getExternalUrl();
        // Some streams may not have the concept of an external URL, so we
        // check here to make sure, since the example assumes this.
        if ($external_url) {
          $url = Url::fromUri($external_url);
          return $url;
        }
      }
      else {
        $url = Url::fromUri($uri);
        // If we did not throw on ::fromUri (you can), we return the URL.
        return $url;
      }
    }
    catch (\Exception $e) {
      return FALSE;
    }
    return FALSE;
  }

}
